import { INode } from './nodes/Node';
export interface Registry<T> {
    register(id: string, value: T): void;
    get(id: string): T;
}
/**
 * Registry for schemas
 */
export declare class SchemaRegistry implements Registry<INode> {
    private registry;
    register(id: string, node: INode): void;
    get(id: string): INode<any>;
}
/**
 * Registry for collections
 */
export declare class CollectionRegistry implements Registry<string[]> {
    private registry;
    register(id: string, list: string[]): void;
    get(id: string): string[];
}
/**
 * Registry for locales
 */
export interface Locale {
    [key: string]: string;
}
export declare class LocaleRegistry implements Registry<Locale> {
    private registry;
    language: string;
    /**
     *
     * @param id locale identifier
     * @param locale object mapping keys to translations
     */
    register(id: string, locale: Locale): void;
    get(id: string): Locale;
    has(id: string): boolean;
    getLocale(key: string): string;
}
export declare const SCHEMAS: SchemaRegistry;
export declare const COLLECTIONS: CollectionRegistry;
export declare const LOCALES: LocaleRegistry;
/**
 * Gets the locale of a key from the locale registry.
 *
 * @param key string that refers to a locale ID.
 * @param params optional parameters
 * @returns the key itself if it isn't found for the selected language
 */
export declare const locale: (key: string, params?: string[]) => string;
