"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RangeNode = void 0;
const Node_1 = require("../../nodes/Node");
const Registries_1 = require("../../Registries");
const NumberNode_1 = require("../../nodes/NumberNode");
exports.RangeNode = (config) => {
    const numberNode = NumberNode_1.NumberNode(config);
    const getState = (el) => {
        var _a, _b, _c, _d, _e;
        const type = (config === null || config === void 0 ? void 0 : config.forceRange) ? 'range' : el.querySelector('select').value;
        if (type === 'exact') {
            return Number(((_a = el.querySelector('input')) === null || _a === void 0 ? void 0 : _a.value) || undefined);
        }
        if (type === 'range') {
            const min = Number(((_b = el.querySelectorAll('input')[0]) === null || _b === void 0 ? void 0 : _b.value) || undefined);
            const max = Number(((_c = el.querySelectorAll('input')[1]) === null || _c === void 0 ? void 0 : _c.value) || undefined);
            return {
                min: isNaN(min) ? undefined : min,
                max: isNaN(max) ? undefined : max
            };
        }
        const n = Number(((_d = el.querySelectorAll('input')[0]) === null || _d === void 0 ? void 0 : _d.value) || undefined);
        const p = Number(((_e = el.querySelectorAll('input')[1]) === null || _e === void 0 ? void 0 : _e.value) || undefined);
        return {
            type: 'binomial',
            n: isNaN(n) ? undefined : n,
            p: isNaN(p) ? undefined : p
        };
    };
    return {
        ...Node_1.Base,
        default: () => (config === null || config === void 0 ? void 0 : config.forceRange) ? {} : 0,
        render(path, value, view, options) {
            var _a, _b, _c;
            let curType = '';
            let input = '';
            if (value === undefined || typeof value === 'number') {
                curType = 'exact';
                input = `<input value=${value === undefined ? '' : value}>`;
            }
            else if (value.type === 'binomial') {
                curType = 'binomial';
                input = `<label>${Registries_1.locale('range.n')}</label>
          <input value=${value.n === undefined ? '' : value.n}>
          <label>${Registries_1.locale('range.p')}</label>
          <input value=${value.p === undefined ? '' : value.p}>`;
            }
            else {
                curType = 'range';
                input = `<label>${Registries_1.locale('range.min')}</label>
          <input value=${value.min === undefined ? '' : value.min}>
          <label>${Registries_1.locale('range.max')}</label>
          <input value=${value.max === undefined ? '' : value.max}>`;
            }
            const id = view.registerChange(el => {
                view.model.set(path, getState(el));
            });
            const selectId = view.register(el => {
                el.value = curType;
                el.addEventListener('change', evt => {
                    const target = el.value;
                    const newValue = target === 'exact' ? this.default() :
                        target === 'binomial' ? { type: 'binomial' } : {};
                    view.model.set(path, newValue);
                    evt.stopPropagation();
                });
            });
            return `<div class="node range-node node-header" data-id="${id}" ${path.error(false)}>
        ${(_a = options === null || options === void 0 ? void 0 : options.prepend) !== null && _a !== void 0 ? _a : ''}
        <label>${(_b = options === null || options === void 0 ? void 0 : options.label) !== null && _b !== void 0 ? _b : path.locale()}</label>
        ${(_c = options === null || options === void 0 ? void 0 : options.inject) !== null && _c !== void 0 ? _c : ''}
        ${(config === null || config === void 0 ? void 0 : config.forceRange) ? `` : `<select data-id="${selectId}">
          <option value="exact">${Registries_1.locale('range.exact')}</option>
          <option value="range">${Registries_1.locale('range.range')}</option>
          ${(config === null || config === void 0 ? void 0 : config.allowBinomial) ? `
            <option value="binomial">${Registries_1.locale('range.binomial')}</option>
          ` : ``}
        </select>`}
        ${input}
      </div>`;
        },
        validate(path, value, errors, options) {
            if (typeof value !== 'number' && typeof value !== 'object') {
                errors.add(path, 'error.expected_range');
                return value;
            }
            if (value === undefined || typeof value === 'number') {
                if (config === null || config === void 0 ? void 0 : config.forceRange) {
                    errors.add(path, 'error.invalid_exact');
                }
                numberNode.validate(path, value, errors, options);
            }
            else if (value.type === 'binomial') {
                if (config === null || config === void 0 ? void 0 : config.allowBinomial) {
                    NumberNode_1.NumberNode({ min: 0, integer: true }).validate(path.push('n'), value.n, errors, options);
                    NumberNode_1.NumberNode({ min: 0, max: 1 }).validate(path.push('p'), value.p, errors, options);
                }
                else {
                    errors.add(path, 'error.invalid_binomial');
                }
            }
            else {
                if (value.min !== undefined) {
                    numberNode.validate(path.push('min'), value.min, errors, options);
                }
                if (value.max !== undefined) {
                    numberNode.validate(path.push('max'), value.max, errors, options);
                }
            }
            return value;
        }
    };
};
