"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AdvancementSchema = exports.PredicateChoice = void 0;
const Node_1 = require("../../nodes/Node");
const EnumNode_1 = require("../../nodes/EnumNode");
const Resource_1 = require("../nodes/Resource");
const NumberNode_1 = require("../../nodes/NumberNode");
const BooleanNode_1 = require("../../nodes/BooleanNode");
const ObjectNode_1 = require("../../nodes/ObjectNode");
const ListNode_1 = require("../../nodes/ListNode");
const RangeNode_1 = require("../nodes/RangeNode");
const MapNode_1 = require("../../nodes/MapNode");
const StringNode_1 = require("../../nodes/StringNode");
const Reference_1 = require("../../nodes/Reference");
const JsonNode_1 = require("../nodes/JsonNode");
const ChoiceNode_1 = require("../../nodes/ChoiceNode");
const Registries_1 = require("../../Registries");
require("./Predicates");
exports.PredicateChoice = (node) => {
    return ChoiceNode_1.ChoiceNode([
        [
            'object',
            node,
            v => { var _a, _b; return (_b = (_a = v[0]) === null || _a === void 0 ? void 0 : _a.predicate) !== null && _b !== void 0 ? _b : ({}); }
        ],
        [
            'list',
            ListNode_1.ListNode(Reference_1.Reference('condition')),
            v => [{
                    condition: 'minecraft:entity_properties',
                    predicate: v
                }]
        ]
    ]);
};
Registries_1.SCHEMAS.register('advancement', Node_1.Mod(ObjectNode_1.ObjectNode({
    display: ObjectNode_1.ObjectNode({
        icon: Node_1.Force(ObjectNode_1.ObjectNode({
            item: Node_1.Force(Resource_1.Resource(EnumNode_1.EnumNode('item'))),
            nbt: StringNode_1.StringNode()
        })),
        title: Node_1.Force(JsonNode_1.JsonNode()),
        description: Node_1.Force(JsonNode_1.JsonNode()),
        background: StringNode_1.StringNode(),
        frame: EnumNode_1.EnumNode(['task', 'challenge', 'goal']),
        show_toast: BooleanNode_1.BooleanNode(),
        announce_to_chat: BooleanNode_1.BooleanNode(),
        hidden: BooleanNode_1.BooleanNode()
    }, { collapse: true }),
    parent: StringNode_1.StringNode(),
    criteria: MapNode_1.MapNode(StringNode_1.StringNode(), Reference_1.Reference('advancement-criteria')),
    requirements: ListNode_1.ListNode(ListNode_1.ListNode(StringNode_1.StringNode())),
    rewards: ObjectNode_1.ObjectNode({
        function: StringNode_1.StringNode(),
        loot: StringNode_1.StringNode(),
        recipes: ListNode_1.ListNode(StringNode_1.StringNode()),
        experience: NumberNode_1.NumberNode({ integer: true })
    }, { collapse: true }),
}, { context: 'advancement' }), {
    default: () => ({
        criteria: {
            requirement: {
                trigger: 'minecraft:location'
            }
        }
    })
}));
Registries_1.SCHEMAS.register('advancement-criteria', ObjectNode_1.ObjectNode({
    trigger: Node_1.Force(Resource_1.Resource(EnumNode_1.EnumNode('advancement_trigger'))),
    conditions: ObjectNode_1.ObjectNode({
        player: Node_1.Mod(exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true })), {
            enabled: (path) => path.pop().push('trigger').get() !== 'minecraft:impossible'
        }),
        [ObjectNode_1.Switch]: path => path.pop().push('trigger'),
        [ObjectNode_1.Case]: {
            'minecraft:bee_nest_destroyed': {
                block: Resource_1.Resource(EnumNode_1.EnumNode('block', { search: true })),
                num_bees_inside: NumberNode_1.NumberNode({ integer: true }),
                item: Reference_1.Reference('item-predicate', { collapse: true })
            },
            'minecraft:bred_animals': {
                parent: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true })),
                partner: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true })),
                child: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true }))
            },
            'minecraft:brewed_potion': {
                potion: StringNode_1.StringNode()
            },
            'minecraft:changed_dimension': {
                from: Resource_1.Resource(StringNode_1.StringNode()),
                to: Resource_1.Resource(StringNode_1.StringNode())
            },
            'minecraft:channeled_lightning': {
                victims: ListNode_1.ListNode(exports.PredicateChoice(Reference_1.Reference('entity-predicate')))
            },
            'minecraft:construct_beacon': {
                level: RangeNode_1.RangeNode()
            },
            'minecraft:consume_item': {
                item: Reference_1.Reference('item-predicate', { collapse: true })
            },
            'minecraft:cured_zombie_villager': {
                villager: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true })),
                zombie: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true }))
            },
            'minecraft:effects_changed': {
                effects: MapNode_1.MapNode(EnumNode_1.EnumNode('mob_effect', { search: true }), ObjectNode_1.ObjectNode({
                    amplifier: RangeNode_1.RangeNode(),
                    duration: RangeNode_1.RangeNode()
                }))
            },
            'minecraft:enter_block': {
                block: Resource_1.Resource(EnumNode_1.EnumNode('block', { search: true })),
                state: MapNode_1.MapNode(StringNode_1.StringNode(), StringNode_1.StringNode())
            },
            'minecraft:enchanted_item': {
                levels: RangeNode_1.RangeNode(),
                item: Reference_1.Reference('item-predicate', { collapse: true })
            },
            'minecraft:entity_hurt_player': {
                damage: Reference_1.Reference('damage-predicate', { collapse: true })
            },
            'minecraft:entity_killed_player': {
                entity: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true })),
                killing_blow: Reference_1.Reference('damage-source-predicate', { collapse: true })
            },
            'minecraft:filled_bucket': {
                item: Reference_1.Reference('item-predicate', { collapse: true })
            },
            'minecraft:fishing_rod_hooked': {
                entity: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true })),
                item: Reference_1.Reference('item-predicate', { collapse: true })
            },
            'minecraft:hero_of_the_village': {
                location: Reference_1.Reference('location-predicate', { collapse: true })
            },
            'minecraft:inventory_changed': {
                slots: ObjectNode_1.ObjectNode({
                    empty: RangeNode_1.RangeNode(),
                    occupied: RangeNode_1.RangeNode(),
                    full: RangeNode_1.RangeNode()
                }),
                items: ListNode_1.ListNode(Reference_1.Reference('item-predicate'))
            },
            'minecraft:item_durability_changed': {
                delta: RangeNode_1.RangeNode(),
                durability: RangeNode_1.RangeNode(),
                item: Reference_1.Reference('item-predicate', { collapse: true })
            },
            'minecraft:item_used_on_block': {
                item: Reference_1.Reference('item-predicate', { collapse: true }),
                location: Reference_1.Reference('location-predicate', { collapse: true })
            },
            'minecraft:killed_by_crossbow': {
                unique_entity_types: RangeNode_1.RangeNode(),
                victims: ListNode_1.ListNode(exports.PredicateChoice(Reference_1.Reference('entity-predicate')))
            },
            'minecraft:levitation': {
                distance: RangeNode_1.RangeNode(),
                duration: RangeNode_1.RangeNode()
            },
            'minecraft:location': {
                location: Reference_1.Reference('location-predicate', { collapse: true })
            },
            'minecraft:nether_travel': {
                distance: RangeNode_1.RangeNode()
            },
            'minecraft:placed_block': {
                block: Resource_1.Resource(EnumNode_1.EnumNode('block', { search: true })),
                state: MapNode_1.MapNode(StringNode_1.StringNode(), StringNode_1.StringNode()),
                item: Reference_1.Reference('item-predicate', { collapse: true }),
                location: Reference_1.Reference('location-predicate', { collapse: true })
            },
            'minecraft:player_generates_container_loot': {
                loot_table: StringNode_1.StringNode()
            },
            'minecraft:player_hurt_entity': {
                damage: Reference_1.Reference('damage-predicate', { collapse: true }),
                entity: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true }))
            },
            'minecraft:player_killed_entity': {
                entity: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true })),
                killing_blow: Reference_1.Reference('damage-source-predicate', { collapse: true })
            },
            'minecraft:recipe_unlocked': {
                recipe: StringNode_1.StringNode()
            },
            'minecraft:slept_in_bed': {
                location: Reference_1.Reference('location-predicate', { collapse: true })
            },
            'minecraft:slide_down_block': {
                block: Resource_1.Resource(EnumNode_1.EnumNode('block', { search: true }))
            },
            'minecraft:shot_crossbow': {
                item: Reference_1.Reference('item-predicate', { collapse: true })
            },
            'minecraft:summoned_entity': {
                entity: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true }))
            },
            'minecraft:tame_animal': {
                entity: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true }))
            },
            'minecraft:target_hit': {
                projectile: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true })),
                shooter: exports.PredicateChoice(Reference_1.Reference('entity-predicate', { collapse: true })),
                signal_strength: RangeNode_1.RangeNode({ integer: true })
            },
            'minecraft:thrown_item_picked_up_by_entity': {
                entity: Reference_1.Reference('entity-predicate', { collapse: true }),
                item: Reference_1.Reference('item-predicate', { collapse: true })
            },
            'minecraft:used_ender_eye': {
                distance: RangeNode_1.RangeNode()
            },
            'minecraft:used_totem': {
                item: Reference_1.Reference('item-predicate', { collapse: true })
            },
            'minecraft:villager_trade': {
                villager: Reference_1.Reference('entity-predicate', { collapse: true }),
                item: Reference_1.Reference('item-predicate', { collapse: true })
            },
            'minecraft:voluntary_exile': {
                location: Reference_1.Reference('location-predicate', { collapse: true })
            }
        }
    }, { context: 'criterion' })
}, { category: 'predicate', context: 'criterion' }));
exports.AdvancementSchema = Registries_1.SCHEMAS.get('advancement');
