"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConditionSchema = void 0;
const EnumNode_1 = require("../../nodes/EnumNode");
const ResourceNode_1 = require("../nodes/ResourceNode");
const NumberNode_1 = require("../../nodes/NumberNode");
const BooleanNode_1 = require("../../nodes/BooleanNode");
const ObjectNode_1 = require("../../nodes/ObjectNode");
const ListNode_1 = require("../../nodes/ListNode");
const RangeNode_1 = require("../nodes/RangeNode");
const MapNode_1 = require("../../nodes/MapNode");
const StringNode_1 = require("../../nodes/StringNode");
const ReferenceNode_1 = require("../../nodes/ReferenceNode");
const Registries_1 = require("../../Registries");
require("./Predicates");
Registries_1.SCHEMAS.register('condition', new ObjectNode_1.ObjectNode({
    condition: new ResourceNode_1.ResourceNode(Registries_1.COLLECTIONS.get('conditions'), { default: () => 'random_chance' }),
    [ObjectNode_1.Switch]: path => path.push('condition').get(),
    [ObjectNode_1.Case]: {
        'alternative': {
            terms: new ListNode_1.ListNode(new ReferenceNode_1.ReferenceNode('condition'))
        },
        'block_state_property': {
            block: new ResourceNode_1.ResourceNode(Registries_1.COLLECTIONS.get('blocks')),
            properties: new MapNode_1.MapNode(new StringNode_1.StringNode(), new StringNode_1.StringNode())
        },
        'damage_source_properties': {
            predicate: new ReferenceNode_1.ReferenceNode('damage-source-predicate')
        },
        'entity_properties': {
            entity: new EnumNode_1.EnumNode(Registries_1.COLLECTIONS.get('entity-sources'), 'this'),
            predicate: new ReferenceNode_1.ReferenceNode('entity-predicate')
        },
        'entity_scores': {
            entity: new EnumNode_1.EnumNode(Registries_1.COLLECTIONS.get('entity-sources'), 'this'),
            scores: new MapNode_1.MapNode(new StringNode_1.StringNode(), new RangeNode_1.RangeNode())
        },
        'inverted': {
            term: new ReferenceNode_1.ReferenceNode('condition')
        },
        'killed_by_player': {
            inverse: new BooleanNode_1.BooleanNode()
        },
        'location_check': {
            offsetX: new NumberNode_1.NumberNode({ integer: true }),
            offsetY: new NumberNode_1.NumberNode({ integer: true }),
            offsetZ: new NumberNode_1.NumberNode({ integer: true }),
            predicate: new ReferenceNode_1.ReferenceNode('location-predicate')
        },
        'match_tool': {
            predicate: new ReferenceNode_1.ReferenceNode('item-predicate')
        },
        'random_chance': {
            chance: new NumberNode_1.NumberNode({ min: 0, max: 1 })
        },
        'random_chance_with_looting': {
            chance: new NumberNode_1.NumberNode({ min: 0, max: 1 }),
            looting_multiplier: new NumberNode_1.NumberNode()
        },
        'requirements': {
            terms: new ListNode_1.ListNode(new ReferenceNode_1.ReferenceNode('condition')),
        },
        'reference': {
            name: new StringNode_1.StringNode()
        },
        'table_bonus': {
            enchantment: new ResourceNode_1.ResourceNode(Registries_1.COLLECTIONS.get('enchantments')),
            chances: new ListNode_1.ListNode(new NumberNode_1.NumberNode({ min: 0, max: 1 }))
        },
        'time_check': {
            value: new RangeNode_1.RangeNode(),
            period: new NumberNode_1.NumberNode()
        },
        'weather_check': {
            raining: new BooleanNode_1.BooleanNode(),
            thrundering: new BooleanNode_1.BooleanNode()
        }
    }
}, {
    category: 'predicate',
    default: () => ({
        condition: 'random_chance',
        chance: 0.5
    })
}));
exports.ConditionSchema = Registries_1.SCHEMAS.get('condition');
