"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConditionSchema = void 0;
const Node_1 = require("../../nodes/Node");
const EnumNode_1 = require("../../nodes/EnumNode");
const Resource_1 = require("../nodes/Resource");
const NumberNode_1 = require("../../nodes/NumberNode");
const BooleanNode_1 = require("../../nodes/BooleanNode");
const ObjectNode_1 = require("../../nodes/ObjectNode");
const ListNode_1 = require("../../nodes/ListNode");
const RangeNode_1 = require("../nodes/RangeNode");
const MapNode_1 = require("../../nodes/MapNode");
const StringNode_1 = require("../../nodes/StringNode");
const Reference_1 = require("../../nodes/Reference");
const ChoiceNode_1 = require("../../nodes/ChoiceNode");
const Registries_1 = require("../../Registries");
require("./Predicates");
Registries_1.SCHEMAS.register('predicate', ChoiceNode_1.ObjectOrList(Reference_1.Reference('condition'), { context: 'condition' }));
Registries_1.SCHEMAS.register('condition', ObjectNode_1.ObjectNode({
    condition: Resource_1.Resource(EnumNode_1.EnumNode('loot_condition_type', 'minecraft:random_chance')),
    [ObjectNode_1.Switch]: path => path.push('condition'),
    [ObjectNode_1.Case]: {
        'minecraft:alternative': {
            terms: Node_1.Force(ListNode_1.ListNode(Reference_1.Reference('condition')))
        },
        'minecraft:block_state_property': {
            block: Resource_1.Resource(EnumNode_1.EnumNode('block')),
            properties: MapNode_1.MapNode(StringNode_1.StringNode(), StringNode_1.StringNode())
        },
        'minecraft:damage_source_properties': {
            predicate: Reference_1.Reference('damage-source-predicate')
        },
        'minecraft:entity_properties': {
            entity: EnumNode_1.EnumNode('entity_source', 'this'),
            predicate: Reference_1.Reference('entity-predicate')
        },
        'minecraft:entity_scores': {
            entity: EnumNode_1.EnumNode('entity_source', 'this'),
            scores: MapNode_1.MapNode(StringNode_1.StringNode(), RangeNode_1.RangeNode({ forceRange: true }))
        },
        'minecraft:inverted': {
            term: Node_1.Force(Reference_1.Reference('condition'))
        },
        'minecraft:killed_by_player': {
            inverse: BooleanNode_1.BooleanNode()
        },
        'minecraft:location_check': {
            offsetX: NumberNode_1.NumberNode({ integer: true }),
            offsetY: NumberNode_1.NumberNode({ integer: true }),
            offsetZ: NumberNode_1.NumberNode({ integer: true }),
            predicate: Reference_1.Reference('location-predicate')
        },
        'minecraft:match_tool': {
            predicate: Reference_1.Reference('item-predicate')
        },
        'minecraft:random_chance': {
            chance: Node_1.Force(NumberNode_1.NumberNode({ min: 0, max: 1 }))
        },
        'minecraft:random_chance_with_looting': {
            chance: NumberNode_1.NumberNode({ min: 0, max: 1 }),
            looting_multiplier: NumberNode_1.NumberNode()
        },
        'minecraft:requirements': {
            terms: ListNode_1.ListNode(Reference_1.Reference('condition')),
        },
        'minecraft:reference': {
            name: Node_1.Force(StringNode_1.StringNode())
        },
        'minecraft:table_bonus': {
            enchantment: Node_1.Force(Resource_1.Resource(EnumNode_1.EnumNode('enchantment'))),
            chances: Node_1.Force(ListNode_1.ListNode(NumberNode_1.NumberNode({ min: 0, max: 1 })))
        },
        'minecraft:time_check': {
            value: Node_1.Force(RangeNode_1.RangeNode()),
            period: NumberNode_1.NumberNode()
        },
        'minecraft:weather_check': {
            raining: BooleanNode_1.BooleanNode(),
            thundering: BooleanNode_1.BooleanNode()
        }
    }
}, { category: 'predicate', context: 'condition' }));
exports.ConditionSchema = Registries_1.SCHEMAS.get('predicate');
