"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DimensionTypeSchema = exports.DimensionSchema = void 0;
const Node_1 = require("../../nodes/Node");
const EnumNode_1 = require("../../nodes/EnumNode");
const Resource_1 = require("../nodes/Resource");
const NumberNode_1 = require("../../nodes/NumberNode");
const BooleanNode_1 = require("../../nodes/BooleanNode");
const ObjectNode_1 = require("../../nodes/ObjectNode");
const ListNode_1 = require("../../nodes/ListNode");
const MapNode_1 = require("../../nodes/MapNode");
const StringNode_1 = require("../../nodes/StringNode");
const Reference_1 = require("../../nodes/Reference");
const Registries_1 = require("../../Registries");
Registries_1.SCHEMAS.register('dimension', Node_1.Mod(ObjectNode_1.ObjectNode({
    type: Node_1.Force(EnumNode_1.EnumNode('dimension_type', { search: true, additional: true })),
    generator: Node_1.Force(ObjectNode_1.ObjectNode({
        type: Resource_1.Resource(EnumNode_1.EnumNode(['minecraft:noise', 'minecraft:flat', 'minecraft:debug'], 'minecraft:noise')),
        seed: NumberNode_1.NumberNode({ integer: true }),
        [ObjectNode_1.Switch]: path => path.push('type'),
        [ObjectNode_1.Case]: {
            'minecraft:noise': {
                biome_source: Node_1.Force(ObjectNode_1.ObjectNode({
                    type: Resource_1.Resource(EnumNode_1.EnumNode('biome_source', 'minecraft:multi_noise')),
                    seed: NumberNode_1.NumberNode({ integer: true }),
                    [ObjectNode_1.Switch]: path => path.push('type'),
                    [ObjectNode_1.Case]: {
                        'minecraft:fixed': {
                            biome: EnumNode_1.EnumNode('biome', 'minecraft:plains')
                        },
                        'minecraft:multi_noise': {
                            preset: EnumNode_1.EnumNode(['nether']),
                            biomes: ListNode_1.ListNode(Reference_1.Reference('generator-biome'))
                        },
                        'minecraft:checkerboard': {
                            biomes: ListNode_1.ListNode(EnumNode_1.EnumNode('biome', 'minecraft:plains'))
                        },
                        'minecraft:vanilla_layered': {
                            large_biomes: BooleanNode_1.BooleanNode()
                        }
                    }
                }, { category: 'predicate', disableSwitchContext: true })),
                settings: ObjectNode_1.ObjectNode({
                    bedrock_roof_position: NumberNode_1.NumberNode({ integer: true }),
                    bedrock_floor_position: NumberNode_1.NumberNode({ integer: true }),
                    sea_level: NumberNode_1.NumberNode({ integer: true }),
                    disable_mob_generation: BooleanNode_1.BooleanNode(),
                    default_block: ObjectNode_1.ObjectNode({
                        Name: StringNode_1.StringNode(),
                        Properties: MapNode_1.MapNode(StringNode_1.StringNode(), StringNode_1.StringNode())
                    }),
                    default_fluid: ObjectNode_1.ObjectNode({
                        Name: StringNode_1.StringNode(),
                        Properties: MapNode_1.MapNode(StringNode_1.StringNode(), StringNode_1.StringNode())
                    }),
                    noise: ObjectNode_1.ObjectNode({
                        density_factor: NumberNode_1.NumberNode(),
                        density_offset: NumberNode_1.NumberNode(),
                        simplex_surface_noise: BooleanNode_1.BooleanNode(),
                        random_density_offset: BooleanNode_1.BooleanNode(),
                        island_noise_override: BooleanNode_1.BooleanNode(),
                        amplified: BooleanNode_1.BooleanNode(),
                        size_horizontal: NumberNode_1.NumberNode({ integer: true }),
                        size_vertical: NumberNode_1.NumberNode({ integer: true }),
                        height: NumberNode_1.NumberNode({ integer: true }),
                        sampling: ObjectNode_1.ObjectNode({
                            xz_scale: NumberNode_1.NumberNode(),
                            y_scale: NumberNode_1.NumberNode(),
                            xz_factor: NumberNode_1.NumberNode(),
                            y_factor: NumberNode_1.NumberNode()
                        }),
                        bottom_slide: ObjectNode_1.ObjectNode({
                            target: NumberNode_1.NumberNode({ integer: true }),
                            size: NumberNode_1.NumberNode({ integer: true }),
                            offset: NumberNode_1.NumberNode({ integer: true })
                        }),
                        top_slide: ObjectNode_1.ObjectNode({
                            target: NumberNode_1.NumberNode({ integer: true }),
                            size: NumberNode_1.NumberNode({ integer: true }),
                            offset: NumberNode_1.NumberNode({ integer: true })
                        })
                    }, { collapse: true }),
                    structures: Reference_1.Reference('generator-structures')
                }, { collapse: true })
            },
            'minecraft:flat': {
                settings: ObjectNode_1.ObjectNode({
                    biome: Resource_1.Resource(EnumNode_1.EnumNode('biome')),
                    layers: Node_1.Force(ListNode_1.ListNode(Reference_1.Reference('generator-layer'))),
                    structures: Reference_1.Reference('generator-structures')
                })
            }
        }
    }, { disableSwitchContext: true }))
}, { context: 'dimension' }), {
    default: () => ({
        type: 'minecraft:overworld'
    })
}));
Registries_1.SCHEMAS.register('generator-biome', Node_1.Mod(ObjectNode_1.ObjectNode({
    biome: Node_1.Force(Resource_1.Resource(EnumNode_1.EnumNode('biome', { search: true }))),
    parameters: ObjectNode_1.ObjectNode({
        altitude: Node_1.Force(NumberNode_1.NumberNode({ min: -1, max: 1 })),
        temperature: Node_1.Force(NumberNode_1.NumberNode({ min: -1, max: 1 })),
        humidity: Node_1.Force(NumberNode_1.NumberNode({ min: -1, max: 1 })),
        weirdness: Node_1.Force(NumberNode_1.NumberNode({ min: -1, max: 1 })),
        offset: Node_1.Force(NumberNode_1.NumberNode({ min: -1, max: 1 }))
    })
}, { context: 'generator_biome' }), {
    default: () => ({
        biome: 'minecraft:plains',
        parameters: {
            altitude: 0,
            temperature: 0,
            humidity: 0,
            weirdness: 0,
            offset: 0
        }
    })
}));
Registries_1.SCHEMAS.register('generator-structures', ObjectNode_1.ObjectNode({
    stronghold: ObjectNode_1.ObjectNode({
        distance: NumberNode_1.NumberNode({ integer: true }),
        spread: NumberNode_1.NumberNode({ integer: true }),
        count: NumberNode_1.NumberNode({ integer: true })
    }, {
        collapse: true
    }),
    structures: MapNode_1.MapNode(EnumNode_1.EnumNode('structure_feature'), Node_1.Mod(ObjectNode_1.ObjectNode({
        spacing: NumberNode_1.NumberNode({ integer: true }),
        separation: NumberNode_1.NumberNode({ integer: true }),
        salt: NumberNode_1.NumberNode({ integer: true })
    }, { context: 'generator_structure' }), {
        default: () => ({
            spacing: 10,
            separation: 5,
            salt: 0
        })
    }))
}));
Registries_1.SCHEMAS.register('generator-layer', Node_1.Mod(ObjectNode_1.ObjectNode({
    block: Node_1.Force(Resource_1.Resource(EnumNode_1.EnumNode('block', { search: true }))),
    height: Node_1.Force(NumberNode_1.NumberNode({ min: 1, integer: true }))
}), {
    default: () => ({
        block: 'stone',
        height: 1
    })
}));
Registries_1.SCHEMAS.register('dimension-type', Node_1.Mod(ObjectNode_1.ObjectNode({
    ultrawarm: Node_1.Force(BooleanNode_1.BooleanNode({ radio: true })),
    natural: Node_1.Force(BooleanNode_1.BooleanNode({ radio: true })),
    shrunk: Node_1.Force(BooleanNode_1.BooleanNode({ radio: true })),
    piglin_safe: Node_1.Force(BooleanNode_1.BooleanNode({ radio: true })),
    respawn_anchor_works: Node_1.Force(BooleanNode_1.BooleanNode({ radio: true })),
    bed_works: Node_1.Force(BooleanNode_1.BooleanNode({ radio: true })),
    has_raids: Node_1.Force(BooleanNode_1.BooleanNode({ radio: true })),
    has_skylight: Node_1.Force(BooleanNode_1.BooleanNode({ radio: true })),
    has_ceiling: Node_1.Force(BooleanNode_1.BooleanNode({ radio: true })),
    ambient_light: Node_1.Force(NumberNode_1.NumberNode()),
    fixed_time: NumberNode_1.NumberNode({ integer: true }),
    logical_height: Node_1.Force(NumberNode_1.NumberNode({ integer: true })),
    infiniburn: Node_1.Force(EnumNode_1.EnumNode([
        'minecraft:infiniburn_overworld',
        'minecraft:infiniburn_nether',
        'minecraft:infiniburn_end'
    ], { search: true, additional: true }))
}, { context: 'dimension_type' }), {
    default: () => ({
        ultrawarm: false,
        natural: true,
        shrunk: false,
        piglin_safe: false,
        respawn_anchor_works: false,
        bed_works: false,
        has_raids: false,
        has_skylight: true,
        has_ceiling: false,
        ambient_light: 0,
        logical_height: 256,
        infiniburn: 'minecraft:infiniburn_overworld',
    })
}));
exports.DimensionSchema = Registries_1.SCHEMAS.get('dimension');
exports.DimensionTypeSchema = Registries_1.SCHEMAS.get('dimension-type');
