"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LootTableSchema = void 0;
const Node_1 = require("../../nodes/Node");
const EnumNode_1 = require("../../nodes/EnumNode");
const Resource_1 = require("../nodes/Resource");
const NumberNode_1 = require("../../nodes/NumberNode");
const BooleanNode_1 = require("../../nodes/BooleanNode");
const ObjectNode_1 = require("../../nodes/ObjectNode");
const ListNode_1 = require("../../nodes/ListNode");
const RangeNode_1 = require("../nodes/RangeNode");
const StringNode_1 = require("../../nodes/StringNode");
const Reference_1 = require("../../nodes/Reference");
const JsonNode_1 = require("../nodes/JsonNode");
const Registries_1 = require("../../Registries");
require("./Condition");
const conditions = {
    conditions: ListNode_1.ListNode(Reference_1.Reference('condition'))
};
const functionsAndConditions = {
    functions: ListNode_1.ListNode(Reference_1.Reference('loot-function')),
    ...conditions
};
Registries_1.SCHEMAS.register('loot-table', Node_1.Mod(ObjectNode_1.ObjectNode({
    pools: Node_1.Force(ListNode_1.ListNode(ObjectNode_1.ObjectNode({
        rolls: Node_1.Force(RangeNode_1.RangeNode({ allowBinomial: true, integer: true, min: 1 })),
        bonus_rolls: RangeNode_1.RangeNode({ allowBinomial: true }),
        entries: ListNode_1.ListNode(Reference_1.Reference('loot-entry')),
        ...functionsAndConditions
    }, { context: 'loot_pool' }))),
    ...functionsAndConditions
}, { context: 'loot_table' }), {
    default: () => ({
        pools: [{
                rolls: 1,
                entries: [{}]
            }]
    })
}));
Registries_1.SCHEMAS.register('loot-entry', ObjectNode_1.ObjectNode({
    type: Resource_1.Resource(EnumNode_1.EnumNode('loot_pool_entry_type', 'minecraft:item')),
    weight: Node_1.Mod(NumberNode_1.NumberNode({ integer: true, min: 1 }), {
        enabled: (path) => {
            var _a;
            return ((_a = path.pop().get()) === null || _a === void 0 ? void 0 : _a.length) > 1
                && !['minecraft:alternatives', 'minecraft:group', 'minecraft:sequence'].includes(path.push('type').get());
        }
    }),
    [ObjectNode_1.Switch]: path => path.push('type'),
    [ObjectNode_1.Case]: {
        'minecraft:alternatives': {
            children: ListNode_1.ListNode(Reference_1.Reference('loot-entry')),
            ...functionsAndConditions
        },
        'minecraft:dynamic': {
            name: Node_1.Force(StringNode_1.StringNode()),
            ...functionsAndConditions
        },
        'minecraft:group': {
            children: ListNode_1.ListNode(Reference_1.Reference('loot-entry')),
            ...functionsAndConditions
        },
        'minecraft:item': {
            name: Node_1.Force(Resource_1.Resource(EnumNode_1.EnumNode('item', { search: true }))),
            ...functionsAndConditions
        },
        'minecraft:loot_table': {
            name: Node_1.Force(StringNode_1.StringNode()),
            ...functionsAndConditions
        },
        'minecraft:sequence': {
            children: ListNode_1.ListNode(Reference_1.Reference('loot-entry')),
            ...functionsAndConditions
        },
        'minecraft:tag': {
            name: Node_1.Force(StringNode_1.StringNode()),
            expand: BooleanNode_1.BooleanNode(),
            ...functionsAndConditions
        }
    }
}, { context: 'loot_entry' }));
Registries_1.SCHEMAS.register('loot-function', ObjectNode_1.ObjectNode({
    function: Resource_1.Resource(EnumNode_1.EnumNode('loot_function_type', 'minecraft:set_count')),
    [ObjectNode_1.Switch]: path => path.push('function'),
    [ObjectNode_1.Case]: {
        'minecraft:apply_bonus': {
            enchantment: Node_1.Force(EnumNode_1.EnumNode('enchantment')),
            formula: Resource_1.Resource(EnumNode_1.EnumNode([
                'minecraft:uniform_bonus_count',
                'minecraft:binomial_with_bonus_count',
                'minecraft:ore_drops'
            ], 'minecraft:uniform_bonus_count')),
            parameters: Node_1.Mod(ObjectNode_1.ObjectNode({
                bonusMultiplier: Node_1.Mod(NumberNode_1.NumberNode(), {
                    enabled: (path) => path.pop().push('formula').get() === 'minecraft:uniform_bonus_count'
                }),
                extra: Node_1.Mod(NumberNode_1.NumberNode(), {
                    enabled: (path) => path.pop().push('formula').get() === 'minecraft:binomial_with_bonus_count'
                }),
                probability: Node_1.Mod(NumberNode_1.NumberNode(), {
                    enabled: (path) => path.pop().push('formula').get() === 'minecraft:binomial_with_bonus_count'
                })
            }), {
                enabled: (path) => path.push('formula').get() !== 'minecraft:ore_drops'
            }),
            ...conditions
        },
        'minecraft:copy_name': {
            source: EnumNode_1.EnumNode('copy_source', 'this'),
            ...conditions
        },
        'minecraft:copy_nbt': {
            source: EnumNode_1.EnumNode('copy_source', 'this'),
            ops: ListNode_1.ListNode(ObjectNode_1.ObjectNode({
                source: Node_1.Force(StringNode_1.StringNode()),
                target: Node_1.Force(StringNode_1.StringNode()),
                op: EnumNode_1.EnumNode(['replace', 'append', 'merge'])
            }, { context: 'nbt_operation' })),
            ...conditions
        },
        'minecraft:copy_state': {
            block: Resource_1.Resource(EnumNode_1.EnumNode('block')),
            properties: ListNode_1.ListNode(StringNode_1.StringNode()),
            ...conditions
        },
        'minecraft:enchant_randomly': {
            enchantments: ListNode_1.ListNode(EnumNode_1.EnumNode('enchantment')),
            ...conditions
        },
        'minecraft:enchant_with_levels': {
            levels: Node_1.Force(RangeNode_1.RangeNode({ allowBinomial: true })),
            treasure: BooleanNode_1.BooleanNode(),
            ...conditions
        },
        'minecraft:exploration_map': {
            destination: Node_1.Force(EnumNode_1.EnumNode('structure_feature')),
            decoration: Node_1.Force(EnumNode_1.EnumNode('map_decoration')),
            zoom: NumberNode_1.NumberNode({ integer: true }),
            search_radius: NumberNode_1.NumberNode({ integer: true }),
            skip_existing_chunks: BooleanNode_1.BooleanNode(),
            ...conditions
        },
        'minecraft:fill_player_head': {
            entity: EnumNode_1.EnumNode('entity_source'),
            ...conditions
        },
        'minecraft:limit_count': {
            limit: RangeNode_1.RangeNode(),
            ...conditions
        },
        'minecraft:looting_enchant': {
            count: RangeNode_1.RangeNode({ allowBinomial: true }),
            limit: NumberNode_1.NumberNode({ integer: true }),
            ...conditions
        },
        'minecraft:set_attributes': {
            modifiers: ListNode_1.ListNode(Reference_1.Reference('attribute-modifier')),
            ...conditions
        },
        'minecraft:set_contents': {
            entries: ListNode_1.ListNode(Reference_1.Reference('loot-entry')),
            ...conditions
        },
        'minecraft:set_count': {
            count: Node_1.Force(RangeNode_1.RangeNode({ allowBinomial: true })),
            ...conditions
        },
        'minecraft:set_damage': {
            damage: Node_1.Force(RangeNode_1.RangeNode({ allowBinomial: true })),
            ...conditions
        },
        'minecraft:set_loot_table': {
            name: Node_1.Force(StringNode_1.StringNode()),
            seed: NumberNode_1.NumberNode({ integer: true })
        },
        'minecraft:set_lore': {
            entity: Node_1.Force(EnumNode_1.EnumNode('entity_source', 'this')),
            lore: Node_1.Force(ListNode_1.ListNode(JsonNode_1.JsonNode())),
            replace: BooleanNode_1.BooleanNode(),
            ...conditions
        },
        'minecraft:set_name': {
            entity: Node_1.Force(EnumNode_1.EnumNode('entity_source', 'this')),
            name: Node_1.Force(JsonNode_1.JsonNode()),
            ...conditions
        },
        'minecraft:set_nbt': {
            tag: Node_1.Force(StringNode_1.StringNode()),
            ...conditions
        },
        'minecraft:set_stew_effect': {
            effects: ListNode_1.ListNode(ObjectNode_1.ObjectNode({
                type: StringNode_1.StringNode(),
                duration: RangeNode_1.RangeNode()
            })),
            ...conditions
        }
    }
}, { category: 'function', context: 'function' }));
Registries_1.SCHEMAS.register('attribute-modifier', ObjectNode_1.ObjectNode({
    attribute: EnumNode_1.EnumNode('attribute'),
    name: StringNode_1.StringNode(),
    amount: RangeNode_1.RangeNode({ allowBinomial: true }),
    operation: EnumNode_1.EnumNode(['addition', 'multiply_base', 'multiply_total']),
    slot: ListNode_1.ListNode(EnumNode_1.EnumNode('slot'))
}, { context: 'attribute_modifier' }));
exports.LootTableSchema = Registries_1.SCHEMAS.get('loot-table');
