"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ObjectNode_1 = require("../../nodes/ObjectNode");
const Resource_1 = require("../nodes/Resource");
const EnumNode_1 = require("../../nodes/EnumNode");
const ListNode_1 = require("../../nodes/ListNode");
const RangeNode_1 = require("../nodes/RangeNode");
const StringNode_1 = require("../../nodes/StringNode");
const Reference_1 = require("../../nodes/Reference");
const BooleanNode_1 = require("../../nodes/BooleanNode");
const MapNode_1 = require("../../nodes/MapNode");
const Registries_1 = require("../../Registries");
require("./Collections");
const Node_1 = require("../../nodes/Node");
Registries_1.SCHEMAS.register('item-predicate', ObjectNode_1.ObjectNode({
    item: Resource_1.Resource(EnumNode_1.EnumNode('item', { search: true })),
    tag: StringNode_1.StringNode(),
    count: RangeNode_1.RangeNode(),
    durability: RangeNode_1.RangeNode(),
    potion: StringNode_1.StringNode(),
    nbt: StringNode_1.StringNode(),
    enchantments: ListNode_1.ListNode(Reference_1.Reference('enchantment-predicate'))
}, { context: 'item' }));
Registries_1.SCHEMAS.register('enchantment-predicate', ObjectNode_1.ObjectNode({
    enchantment: Resource_1.Resource(EnumNode_1.EnumNode('enchantment', { search: true })),
    levels: RangeNode_1.RangeNode()
}, { context: 'enchantment' }));
Registries_1.SCHEMAS.register('block-predicate', ObjectNode_1.ObjectNode({
    block: Resource_1.Resource(EnumNode_1.EnumNode('block', { search: true })),
    tag: StringNode_1.StringNode(),
    nbt: StringNode_1.StringNode(),
    state: MapNode_1.MapNode(StringNode_1.StringNode(), StringNode_1.StringNode())
}, { context: 'block' }));
Registries_1.SCHEMAS.register('fluid-predicate', ObjectNode_1.ObjectNode({
    fluid: Resource_1.Resource(EnumNode_1.EnumNode('fluid', { search: true })),
    tag: StringNode_1.StringNode(),
    nbt: StringNode_1.StringNode(),
    state: MapNode_1.MapNode(StringNode_1.StringNode(), StringNode_1.StringNode())
}, { context: 'fluid' }));
Registries_1.SCHEMAS.register('location-predicate', ObjectNode_1.ObjectNode({
    position: ObjectNode_1.ObjectNode({
        x: RangeNode_1.RangeNode(),
        y: RangeNode_1.RangeNode(),
        z: RangeNode_1.RangeNode()
    }, { collapse: true }),
    biome: Resource_1.Resource(EnumNode_1.EnumNode('biome', { search: true })),
    feature: Resource_1.Resource(EnumNode_1.EnumNode('structure_feature', { search: true })),
    dimension: Resource_1.Resource(EnumNode_1.EnumNode('dimension', { search: true, additional: true })),
    light: ObjectNode_1.ObjectNode({
        light: RangeNode_1.RangeNode()
    }),
    smokey: BooleanNode_1.BooleanNode(),
    block: Reference_1.Reference('block-predicate', { collapse: true }),
    fluid: Reference_1.Reference('fluid-predicate', { collapse: true })
}, { context: 'location' }));
Registries_1.SCHEMAS.register('statistic-predicate', ObjectNode_1.ObjectNode({
    type: EnumNode_1.EnumNode('stat_type', 'minecraft:custom'),
    stat: Node_1.Force(StringNode_1.StringNode()),
    value: Node_1.Force(RangeNode_1.RangeNode())
}));
Registries_1.SCHEMAS.register('player-predicate', ObjectNode_1.ObjectNode({
    gamemode: EnumNode_1.EnumNode('gamemode'),
    level: RangeNode_1.RangeNode(),
    advancements: MapNode_1.MapNode(StringNode_1.StringNode(), BooleanNode_1.BooleanNode()),
    recipes: MapNode_1.MapNode(StringNode_1.StringNode(), BooleanNode_1.BooleanNode()),
    stats: ListNode_1.ListNode(Reference_1.Reference('statistic-predicate'))
}, { context: 'player' }));
Registries_1.SCHEMAS.register('status-effect-predicate', ObjectNode_1.ObjectNode({
    amplifier: RangeNode_1.RangeNode(),
    duration: RangeNode_1.RangeNode(),
    ambient: BooleanNode_1.BooleanNode(),
    visible: BooleanNode_1.BooleanNode()
}, { context: 'status_effect' }));
Registries_1.SCHEMAS.register('distance-predicate', ObjectNode_1.ObjectNode({
    x: RangeNode_1.RangeNode(),
    y: RangeNode_1.RangeNode(),
    z: RangeNode_1.RangeNode(),
    absolute: RangeNode_1.RangeNode(),
    horizontal: RangeNode_1.RangeNode()
}, { context: 'distance' }));
Registries_1.SCHEMAS.register('entity-predicate', ObjectNode_1.ObjectNode({
    type: StringNode_1.StringNode(),
    nbt: StringNode_1.StringNode(),
    team: StringNode_1.StringNode(),
    location: Reference_1.Reference('location-predicate', { collapse: true }),
    distance: Reference_1.Reference('distance-predicate', { collapse: true }),
    flags: ObjectNode_1.ObjectNode({
        is_on_fire: BooleanNode_1.BooleanNode(),
        is_sneaking: BooleanNode_1.BooleanNode(),
        is_sprinting: BooleanNode_1.BooleanNode(),
        is_swimming: BooleanNode_1.BooleanNode(),
        is_baby: BooleanNode_1.BooleanNode()
    }, { collapse: true }),
    equipment: MapNode_1.MapNode(EnumNode_1.EnumNode('slot', 'mainhand'), Reference_1.Reference('item-predicate')),
    vehicle: Reference_1.Reference('entity-predicate', { collapse: true }),
    targeted_entity: Reference_1.Reference('entity-predicate', { collapse: true }),
    player: Reference_1.Reference('player-predicate', { collapse: true }),
    fishing_hook: ObjectNode_1.ObjectNode({
        in_open_water: BooleanNode_1.BooleanNode()
    }),
    effects: MapNode_1.MapNode(Resource_1.Resource(EnumNode_1.EnumNode('mob_effect', { search: true })), Reference_1.Reference('status-effect-predicate'))
}, { context: 'entity' }));
Registries_1.SCHEMAS.register('damage-source-predicate', ObjectNode_1.ObjectNode({
    is_explosion: BooleanNode_1.BooleanNode(),
    is_fire: BooleanNode_1.BooleanNode(),
    is_magic: BooleanNode_1.BooleanNode(),
    is_projectile: BooleanNode_1.BooleanNode(),
    is_lightning: BooleanNode_1.BooleanNode(),
    bypasses_armor: BooleanNode_1.BooleanNode(),
    bypasses_invulnerability: BooleanNode_1.BooleanNode(),
    bypasses_magic: BooleanNode_1.BooleanNode(),
    source_entity: Reference_1.Reference('entity-predicate', { collapse: true }),
    direct_entity: Reference_1.Reference('entity-predicate', { collapse: true })
}, { context: 'damage_source' }));
Registries_1.SCHEMAS.register('damage-predicate', ObjectNode_1.ObjectNode({
    dealt: RangeNode_1.RangeNode(),
    taken: RangeNode_1.RangeNode(),
    blocked: BooleanNode_1.BooleanNode(),
    source_entity: Reference_1.Reference('entity-predicate', { collapse: true }),
    type: Reference_1.Reference('damage-source-predicate', { collapse: true })
}, { context: 'damage' }));
