import { Path } from './Path';
import { INode } from '../nodes/Node';
import { Errors } from './Errors';
export declare type ModelListener = {
    invalidated?(model: DataModel): void;
    errors?(errors: Errors): void;
};
declare type DataModelOptions = {
    historyMax?: number;
};
/**
 * Holding the data linked to a given schema
 */
export declare class DataModel {
    data: any;
    schema: INode;
    /** A list of listeners that want to be notified when the model is invalidated */
    listeners: ModelListener[];
    errors: Errors;
    history: string[];
    historyIndex: number;
    historyMax: number;
    /**
     * @param schema node to use as schema for this model
     */
    constructor(schema: INode, options?: DataModelOptions);
    /**
     * Adds a listener to notify when the model is invalidated
     * @param listener the listener to be notified
     */
    addListener(listener: ModelListener): void;
    /**
     * Removes a listener from this model
     * @param listener the listener to be removed
     */
    removeListener(listener: ModelListener): void;
    /**
     * Updates the history and notifies all
     * listeners that the model is invalidated
     */
    invalidate(): void;
    /**
     * Notifies all listeners that the model is invalidated
     */
    silentInvalidate(): void;
    /**
     * Resets the full data and notifies listeners
     * @param value new model data
     */
    reset(value: any, loose?: boolean): void;
    /**
     * Gets the data at a specified path
     * @param path path at which to find the data
     * @returns undefined, if the the path does not exist in the data
     */
    get(path: Path): any;
    /**
     * Updates the date on a path. Node will be removed when value is undefined
     * @param path path to update
     * @param value new data at the specified path
     */
    set(path: Path, value: any): void;
    /**
     * Go one step back in history
     */
    undo(): void;
    /**
     * Go one step forward in history.
     */
    redo(): void;
    /**
     * Uses the schema to check whether the data is valid
     */
    validate(loose?: boolean): void;
    error(path: Path, error: string, ...params: any): void;
}
export {};
