import { DataModel } from './DataModel';
export declare type PathElement = number | string;
/**
 * Immutable helper class to represent a path in data
 */
export declare class Path {
    modelArr: PathElement[];
    localeArr: string[];
    model?: DataModel;
    /**
     * @param modelArr Initial array of path model elements. Empty if not given
     * @param localeArr Initial array of path locale elements. Empty if not given
     * @param model Model attached to this path
     */
    constructor(modelArr?: PathElement[], localeArr?: string[], model?: DataModel);
    /**
     * The last model element of this path
     */
    last(): PathElement;
    /**
     * A new path with the last model element removed
     */
    pop(): Path;
    /**
     * A new path with an element added at the end
     * @param element element to push at the end of the array
     */
    push(element: PathElement): Path;
    /**
     * Push an element exclusivly to the model array
     * @param element
     */
    modelPush(element: PathElement): Path;
    /**
     * Push an element exclusivly to the locale array
     * @param element
     */
    localePush(element: PathElement): Path;
    copy(): Path;
    getArray(): PathElement[];
    /**
     * Attaches a model to this path and all paths created from this
     * @param model
     */
    withModel(model: DataModel): Path;
    getModel(): DataModel | undefined;
    /**
     * Gets the data from the model if it was attached
     * @returns undefined, if no model was attached
     */
    get(): any;
    /**
     * Sets the value to the model if it was attached
     */
    set(value: any): void;
    /**
     * Gets the locale of a key from the locale registry.
     *
     * @param key string or path that refers to a locale ID.
     *    If a string is given, an exact match is required.
     *    If a path is given, it finds the longest match at the end.
     * @returns undefined if the key isn't found for the selected language
     */
    locale: () => string;
    /**
     * Gets the error inside this path if the model is attached
     * @returns a html attribute containing the error message
     */
    error(exact?: boolean): string;
    /**
     * Checks path equality
     * @param other path to compare
     */
    equals(other: Path): boolean;
    /**
     * Checks if this path is inside another path
     * @param other parent path where this path should be inside
     */
    inside(other: Path): boolean;
    toString(): string;
    forEach(fn: (value: PathElement, index: number, array: PathElement[]) => void, thisArg?: any): void;
}
