"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BooleanNode = void 0;
const Node_1 = require("./Node");
const Registries_1 = require("../Registries");
/**
 * Boolean node with two buttons for true/false
 */
exports.BooleanNode = (config) => {
    return {
        ...Node_1.Base,
        default: () => false,
        render(path, value, view, options) {
            const onFalse = view.registerClick(el => {
                view.model.set(path, !(config === null || config === void 0 ? void 0 : config.radio) && value === false ? undefined : false);
            });
            const onTrue = view.registerClick(el => {
                view.model.set(path, !(config === null || config === void 0 ? void 0 : config.radio) && value === true ? undefined : true);
            });
            return `<div class="node boolean-node node-header" ${path.error()}>
        ${(options === null || options === void 0 ? void 0 : options.removeId) ? `<button data-id="${options === null || options === void 0 ? void 0 : options.removeId}"></button>` : ``}
        ${(options === null || options === void 0 ? void 0 : options.hideLabel) ? `` : `<label>${path.locale()}</label>`}
        <button${value === false ? ' class="selected"' : ' '} 
          data-id="${onFalse}">${Registries_1.locale('false')}</button>
        <button${value === true ? ' class="selected"' : ' '} 
          data-id="${onTrue}">${Registries_1.locale('true')}</button>
      </div>`;
        },
        validate(path, value, errors) {
            if (typeof value !== 'boolean' || value === undefined) {
                errors.add(path, 'error.expected_boolean');
            }
            return value;
        }
    };
};
