"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectOrList = exports.ChoiceNode = void 0;
const Node_1 = require("./Node");
const Path_1 = require("../model/Path");
const ListNode_1 = require("./ListNode");
/**
 * Node that allows multiple types
 */
exports.ChoiceNode = (choices, config) => {
    const isValid = (choice, value) => {
        switch (choice) {
            case 'list': return value instanceof Array;
            case 'object': return typeof value === 'object' && !(value instanceof Array);
            default: return typeof value === choice;
        }
    };
    const activeChoice = (value) => {
        const index = choices.map(choice => isValid(choice[0], value)).indexOf(true);
        if (index === -1)
            return undefined;
        return choices[index];
    };
    return {
        ...Node_1.Base,
        default: () => choices[0][1].default(),
        transform(path, value, view) {
            const choice = activeChoice(value);
            if (choice === undefined) {
                return value;
            }
            return choice[1].transform(path, value, view);
        },
        render(path, value, view, options) {
            var _a, _b;
            const choice = (_a = activeChoice(value)) !== null && _a !== void 0 ? _a : choices[0];
            const pathWithContext = (config === null || config === void 0 ? void 0 : config.context) ?
                new Path_1.Path(path.getArray(), [config.context], path.getModel()) : path;
            let inject = choices.map(c => {
                if (c[0] === choice[0]) {
                    return `<button class="selected" disabled>${pathWithContext.push(c[0]).locale()}</button>`;
                }
                const buttonId = view.registerClick(el => {
                    view.model.set(path, c[2](value));
                });
                return `<button data-id="${buttonId}">${pathWithContext.push(c[0]).locale()}</button>`;
            }).join('');
            return (_b = choice[1]) === null || _b === void 0 ? void 0 : _b.render(pathWithContext, value, view, {
                ...options,
                label: (options === null || options === void 0 ? void 0 : options.hideHeader) ? '' : undefined,
                hideHeader: false,
                inject
            });
        },
        validate(path, value, errors, options) {
            const choice = activeChoice(value);
            if (choice === undefined) {
                return value;
            }
            return choice[1].validate(path, value, errors, options);
        }
    };
};
exports.ObjectOrList = (node, config) => {
    return exports.ChoiceNode([
        ['object', node, v => v[0]],
        ['list', ListNode_1.ListNode(node), v => [v]]
    ], config);
};
