"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnumNode = void 0;
const Node_1 = require("./Node");
const Path_1 = require("../model/Path");
const Registries_1 = require("../Registries");
const TreeView_1 = require("../view/TreeView");
exports.EnumNode = (values, config) => {
    const getValues = (typeof values === 'string') ?
        () => Registries_1.COLLECTIONS.get(values) :
        () => values;
    const defaultValue = (typeof config === 'string') ? config : undefined;
    const search = (typeof config === 'string') ? undefined : config === null || config === void 0 ? void 0 : config.search;
    const additional = (typeof config === 'string') ? undefined : config === null || config === void 0 ? void 0 : config.additional;
    return {
        ...Node_1.Base,
        default() {
            return defaultValue !== null && defaultValue !== void 0 ? defaultValue : '';
        },
        force() {
            return defaultValue !== undefined;
        },
        render(path, value, view, options) {
            var _a, _b, _c;
            const inputId = view.register(el => {
                el.value = value !== null && value !== void 0 ? value : '';
                el.addEventListener('change', evt => {
                    const newValue = el.value;
                    view.model.set(path, newValue.length === 0 ? undefined : newValue);
                    evt.stopPropagation();
                });
            });
            return `<div class="node enum-node node-header" ${path.error()}>
        ${(_a = options === null || options === void 0 ? void 0 : options.prepend) !== null && _a !== void 0 ? _a : ''}
        <label>${(_b = options === null || options === void 0 ? void 0 : options.label) !== null && _b !== void 0 ? _b : path.locale()}</label>
        ${(_c = options === null || options === void 0 ? void 0 : options.inject) !== null && _c !== void 0 ? _c : ''}
        ${this.renderRaw(path, view, inputId)}
      </div>`;
        },
        validate(path, value, errors, options) {
            if (options.loose && value === undefined && defaultValue !== undefined) {
                return defaultValue;
            }
            if (typeof value !== 'string') {
                errors.add(path, 'error.expected_string');
            }
            else if (!additional && !getValues().includes(value)) {
                errors.add(path, 'error.invalid_enum_option', value);
            }
            return value;
        },
        renderRaw(path, view, inputId) {
            const valuesList = getValues();
            inputId = inputId !== null && inputId !== void 0 ? inputId : view.register(el => {
                el.value = defaultValue !== null && defaultValue !== void 0 ? defaultValue : '';
            });
            if (search) {
                const datalistId = TreeView_1.getId();
                return `<input list="${datalistId}" data-id="${inputId}">
        <datalist id="${datalistId}">
          ${valuesList.map(v => `<option value="${v}">`).join('')}
        </datalist>`;
            }
            const pathWithContext = (typeof values === 'string') ?
                new Path_1.Path(path.getArray(), [values], path.getModel()) : path;
            return `<select data-id="${inputId}">
        ${defaultValue ? `` : `<option value="">${Registries_1.locale('unset')}</option>`}
        ${valuesList.map(v => `<option value="${v}">${pathWithContext.push(v).locale()}</option>`).join('')}
      </select>`;
        },
        getState(el) {
            return el.getElementsByTagName(search ? 'input' : 'select')[0].value;
        }
    };
};
