"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MapNode = void 0;
const Node_1 = require("./Node");
/**
 * Map nodes similar to list nodes, but a string key is required to add children
 */
exports.MapNode = (keys, children) => {
    return {
        ...Node_1.Base,
        transform(path, value, view) {
            if (value === undefined)
                return undefined;
            let res = {};
            Object.keys(value).forEach(f => res[f] = children.transform(path.push(f), value[f], view));
            return res;
        },
        render(path, value, view, options) {
            var _a, _b, _c;
            const onAdd = view.registerClick(el => {
                const key = keys.getState(el.parentElement);
                view.model.set(path.push(key), children.default());
            });
            return `<div class="node map-node">
        <div class="node-header">
          ${(_a = options === null || options === void 0 ? void 0 : options.prepend) !== null && _a !== void 0 ? _a : ''}
          <label>${(_b = options === null || options === void 0 ? void 0 : options.label) !== null && _b !== void 0 ? _b : path.locale()}</label>
          ${(_c = options === null || options === void 0 ? void 0 : options.inject) !== null && _c !== void 0 ? _c : ''}
          ${keys.renderRaw(path, view)}
          <button class="add" data-id="${onAdd}"></button>
        </div>
        <div class="node-body">
          ${Object.keys(value !== null && value !== void 0 ? value : []).map(key => {
                const removeId = view.registerClick(el => view.model.set(path.push(key), undefined));
                return `<div class="node-entry">
            ${children.render(path.modelPush(key), value[key], view, {
                    prepend: `<button class="remove" data-id="${removeId}"></button>`,
                    label: key
                })}
          </div>`;
            }).join('')}
        </div>
      </div>`;
        },
        validate(path, value, errors, options) {
            if (value === null || typeof value !== 'object') {
                errors.add(path, 'error.expected_object');
                return value;
            }
            let res = {};
            Object.keys(value).forEach(k => {
                keys.validate(path, k, errors, options);
                res[k] = children.validate(path.push(k), value[k], errors, options);
            });
            return res;
        }
    };
};
