"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectNode = exports.Case = exports.Switch = void 0;
const Node_1 = require("./Node");
const Path_1 = require("../model/Path");
const Errors_1 = require("../model/Errors");
exports.Switch = Symbol('switch');
exports.Case = Symbol('case');
exports.ObjectNode = (fields, config) => {
    const { [exports.Switch]: filter, [exports.Case]: cases, ...defaultFields } = fields;
    const getActiveFields = (path, model) => {
        if (filter === undefined)
            return defaultFields;
        const switchValue = filter(path).get();
        const activeCase = cases[switchValue];
        return { ...defaultFields, ...activeCase };
    };
    const renderFields = (path, value, view) => {
        value = value !== null && value !== void 0 ? value : {};
        const filterPath = filter ? filter(path) : new Path_1.Path();
        const switchValue = filterPath.get();
        const caseFields = filter ? cases[switchValue] : {};
        const activeFields = filter ? { ...defaultFields, ...caseFields } : defaultFields;
        const filteredKeys = caseFields ? Object.keys(caseFields) : [];
        return Object.keys(activeFields).map(k => {
            if (!activeFields[k].enabled(path, view.model))
                return '';
            const pathWithContext = (config === null || config === void 0 ? void 0 : config.context) ?
                new Path_1.Path(path.getArray(), [config.context], path.getModel()) : path;
            const pathWithFilter = !(config === null || config === void 0 ? void 0 : config.disableSwitchContext) && switchValue && filteredKeys.includes(k) ?
                pathWithContext.localePush(switchValue) : pathWithContext;
            return activeFields[k].render(pathWithFilter.push(k), value[k], view);
        }).join('');
    };
    return ({
        ...Node_1.Base,
        default: () => ({}),
        transform(path, value, view) {
            if (value === undefined || value === null || typeof value !== 'object') {
                return value;
            }
            let res = {};
            const activeFields = getActiveFields(path, view.model);
            Object.keys(activeFields).forEach(f => {
                return res[f] = activeFields[f].transform(path.push(f), value[f], view);
            });
            return res;
        },
        render(path, value, view, options) {
            var _a, _b;
            return `<div class="node object-node"${(config === null || config === void 0 ? void 0 : config.category) ? `data-category="${config === null || config === void 0 ? void 0 : config.category}"` : ''}>
        ${(options === null || options === void 0 ? void 0 : options.hideHeader) ? '' : `<div class="node-header" ${path.error()}>
          ${(options === null || options === void 0 ? void 0 : options.prepend) ? options.prepend : `
            ${((options === null || options === void 0 ? void 0 : options.collapse) || (config === null || config === void 0 ? void 0 : config.collapse)) ? value === undefined ? `
              <button class="collapse closed" data-id="${view.registerClick(() => view.model.set(path, this.default()))}"></button>
            ` : `
              <button class="collapse open" data-id="${view.registerClick(() => view.model.set(path, undefined))}"></button>
            ` : ``}
          `}
          <label>${(_a = options === null || options === void 0 ? void 0 : options.label) !== null && _a !== void 0 ? _a : path.locale()}</label>
          ${(_b = options === null || options === void 0 ? void 0 : options.inject) !== null && _b !== void 0 ? _b : ''}
        </div>`}
        ${(typeof value !== 'object' && value !== undefined) || (((options === null || options === void 0 ? void 0 : options.collapse) || (config === null || config === void 0 ? void 0 : config.collapse)) && value === undefined) ? `` : `
          <div class="node-body">
            ${renderFields(path, value, view)}
          </div>
        `}
      </div>`;
        },
        validate(path, value, errors, options) {
            if (options.loose && value === undefined) {
                value = {};
            }
            if (typeof value !== 'object') {
                errors.add(path, 'error.expected_object');
                return value;
            }
            let activeFields = defaultFields;
            if (filter) {
                const filterPath = filter(path);
                let switchValue = filterPath.get();
                if (path.equals(filterPath.pop())) {
                    const filterField = filterPath.last();
                    switchValue = defaultFields[filterField].validate(path.push(filterField), value[filterField], new Errors_1.Errors(), options);
                }
                activeFields = { ...activeFields, ...cases[switchValue] };
            }
            const activeKeys = Object.keys(activeFields);
            const forcedKeys = activeKeys.filter(k => activeFields[k].force());
            const keys = new Set([...forcedKeys, ...Object.keys(value)]);
            let res = {};
            keys.forEach(k => {
                if (activeKeys.includes(k)) {
                    const newValue = activeFields[k].validate(path.push(k), value[k], errors, options);
                    if (newValue !== undefined) {
                        res[k] = newValue;
                    }
                }
                else {
                    res[k] = value[k];
                }
            });
            if (!(config === null || config === void 0 ? void 0 : config.allowEmpty) && Object.keys(res).length === 0
                && !((config === null || config === void 0 ? void 0 : config.collapse) || options.collapse)) {
                if (options.loose) {
                    return undefined;
                }
            }
            return res;
        }
    });
};
