"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StringNode = void 0;
const Node_1 = require("./Node");
const Registries_1 = require("../Registries");
/**
 * Simple string node with one text field
 */
exports.StringNode = (config) => {
    return {
        ...Node_1.Base,
        default: () => '',
        render(path, value, view, options) {
            var _a, _b, _c;
            const onChange = view.registerChange(el => {
                const value = el.value;
                view.model.set(path, value);
            });
            return `<div class="node string-node node-header" ${path.error()}>
        ${(_a = options === null || options === void 0 ? void 0 : options.prepend) !== null && _a !== void 0 ? _a : ''}
        <label>${(_b = options === null || options === void 0 ? void 0 : options.label) !== null && _b !== void 0 ? _b : path.locale()}</label>
        ${(_c = options === null || options === void 0 ? void 0 : options.inject) !== null && _c !== void 0 ? _c : ''}
        <input data-id="${onChange}" value="${value !== null && value !== void 0 ? value : ''}">
      </div>`;
        },
        validate(path, value, errors, options) {
            var _a;
            if (typeof value !== 'string') {
                errors.add(path, 'error.expected_string');
                return value;
            }
            if (!(config === null || config === void 0 ? void 0 : config.allowEmpty) && value.length === 0) {
                if (options.loose) {
                    return undefined;
                }
                else {
                    errors.add(path, 'error.invalid_empty_string');
                }
            }
            if ((config === null || config === void 0 ? void 0 : config.pattern) && !value.match(config.pattern)) {
                errors.add(path, 'error.invalid_pattern', Registries_1.locale((_a = config.patternError) !== null && _a !== void 0 ? _a : 'pattern'));
            }
            return value;
        },
        getState(el) {
            return el.getElementsByTagName('input')[0].value;
        },
        renderRaw() {
            return `<input>`;
        }
    };
};
