"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const power_assert_1 = __importDefault(require("power-assert"));
const mocha_1 = require("mocha");
const Path_1 = require("../model/Path");
const DataModel_1 = require("../model/DataModel");
const StringNode_1 = require("../nodes/StringNode");
const ObjectNode_1 = require("../nodes/ObjectNode");
mocha_1.describe('Path Tests', () => {
    mocha_1.describe('last() Tests', () => {
        mocha_1.it('should return the last element', () => {
            const path = new Path_1.Path().push('foo');
            power_assert_1.default(path.last() === 'foo');
            power_assert_1.default(path.push('bar').last() === 'bar');
            power_assert_1.default(path.push('bar').pop().last() === 'foo');
        });
        mocha_1.it('should return undefined', () => {
            const path = new Path_1.Path();
            power_assert_1.default(path.last() === undefined);
        });
    });
    mocha_1.describe('Model Tests', () => {
        mocha_1.it('should attach the model', () => {
            const path = new Path_1.Path();
            const schema = new StringNode_1.StringNode();
            const model = new DataModel_1.DataModel(schema);
            const pathWithModel = path.withModel(model);
            power_assert_1.default(pathWithModel.getModel() === model);
        });
        mocha_1.it('should get a path from the model', () => {
            const path = new Path_1.Path();
            const schema = new ObjectNode_1.ObjectNode({
                foo: new StringNode_1.StringNode()
            }, {
                default: () => ({ foo: 'value' })
            });
            const model = new DataModel_1.DataModel(schema);
            const pathWithModel = path.withModel(model);
            power_assert_1.default(pathWithModel.push('foo').get() === 'value');
        });
    });
});
