"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SourceView = void 0;
const Path_1 = require("../model/Path");
const AbstractView_1 = require("./AbstractView");
/**
 * JSON representation view of the model.
 * Renders the result in a <textarea>.
 */
class SourceView extends AbstractView_1.AbstractView {
    /**
     * @param model data model this view represents and listens to
     * @param target DOM element to render the view
     * @param options optional options for the view
     */
    constructor(model, target, options) {
        super(model);
        this.target = target;
        this.options = options;
        this.target.addEventListener('change', evt => this.updateModel());
    }
    invalidated() {
        var _a;
        const transformed = this.model.schema.transform(new Path_1.Path().withModel(this.model), this.model.data, this);
        this.target.value = JSON.stringify(transformed, null, (_a = this.options) === null || _a === void 0 ? void 0 : _a.indentation);
    }
    updateModel() {
        let parsed = {};
        try {
            parsed = JSON.parse(this.target.value);
        }
        catch (err) {
            this.model.error(new Path_1.Path().push('JSON'), err.message);
            return;
        }
        this.model.reset(parsed);
    }
}
exports.SourceView = SourceView;
