import { DataModel } from '../model/DataModel';
import { AbstractView } from './AbstractView';
declare type Registry = {
    [id: string]: (el: Element) => void;
};
/**
 * Helper function to generate a random ID
 */
export declare function getId(): string;
declare type TreeViewOptions = {
    showErrors?: boolean;
    observer?: (el: HTMLElement) => void;
};
/**
 * DOM representation view of the model.
 */
export declare class TreeView extends AbstractView {
    target: HTMLElement;
    registry: Registry;
    showErrors: boolean;
    observer: (el: HTMLElement) => void;
    /**
     * @param model data model this view represents and listens to
     * @param target DOM element to render the view
     */
    constructor(model: DataModel, target: HTMLElement, options?: TreeViewOptions);
    /**
     * Registers a callback and gives an ID
     * @param callback function that is called when the element is mounted
     * @returns the ID that should be applied to the data-id attribute
     */
    register(callback: (el: Element) => void): string;
    /**
     * Registers an event and gives an ID
     * @param type event type
     * @param callback function that is called when the event is fired
     * @returns the ID that should be applied to the data-id attribute
     */
    registerEvent(type: string, callback: (el: Element) => void): string;
    /**
     * Registers a change event and gives an ID
     * @param callback function that is called when the event is fired
     * @returns the ID that should be applied to the data-id attribute
     */
    registerChange(callback: (el: Element) => void): string;
    /**
     * Registers a click event and gives an ID
     * @param callback function that is called when the event is fired
     * @returns the ID that should be applied to the data-id attribute
     */
    registerClick(callback: (el: Element) => void): string;
    /**
     * @override
     */
    invalidated(): void;
}
export {};
